@@ -3,10 +3,12 @@ This is the state file for the Talk2AIAgents4Pharma agent.
 """
 
 from ...talk2biomodels.states.state_talk2biomodels import Talk2Biomodels
-from ...talk2knowledgegraphs.states.state_talk2knowledgegraphs import Talk2KnowledgeGraphs
+from ...talk2knowledgegraphs.states.state_talk2knowledgegraphs import (
+    Talk2KnowledgeGraphs,
+)
 
-class Talk2AIAgents4Pharma(Talk2Biomodels,
-                           Talk2KnowledgeGraphs):
+
+class Talk2AIAgents4Pharma(Talk2Biomodels, Talk2KnowledgeGraphs):
     """
     The state for the Talk2AIAgents4Pharma agent.
 
