@@ -2,12 +2,14 @@
 Import all the modules in the package
 """
 
-from . import search
-from . import single_paper_recommendation
-from . import multi_paper_recommendation
-from . import question_and_answer
-from . import zotero_read
-from . import zotero_write
+from . import (
+    multi_paper_recommendation,
+    question_and_answer,
+    search,
+    single_paper_recommendation,
+    zotero_read,
+    zotero_write,
+)
 
 __all__ = [
     "search",
