@@ -2,12 +2,14 @@
 This file is used to import all the modules in the package.
 """
 
-from . import display_dataframe
-from . import multi_paper_rec
-from . import search
-from . import single_paper_rec
-from . import query_dataframe
-from . import retrieve_semantic_scholar_paper_id
+from . import (
+    display_dataframe,
+    multi_paper_rec,
+    query_dataframe,
+    retrieve_semantic_scholar_paper_id,
+    search,
+    single_paper_rec,
+)
 
 __all__ = [
     "display_dataframe",
