@@ -1,7 +1,5 @@
 """This module contains utility functions for the Semantic Scholar search tool."""
 
-from . import search_helper
-from . import single_helper
-from . import multi_helper
+from . import multi_helper, search_helper, single_helper
 
 __all__ = ["search_helper", "single_helper", "multi_helper"]
